<?php
// Include configuration and utility functions
require_once __DIR__ . '/config.php';
require_once __DIR__ . '/includes/utils.php';

// ---------- 1. Refresh Access Token ----------
function refreshToken() {
    echo "🔄 Attempting to refresh access token...<br>";
    logMessage("🔄 Attempting to refresh access token...");

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, 'https://www.ssgenius.com/mautic-data-fetcher/token_handler.php');
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($ch, CURLOPT_TIMEOUT, 30);

    $response = curl_exec($ch);
    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    $error = curl_error($ch);
    curl_close($ch);

    if ($httpCode === 200 && strpos($response, 'Authorization successful') !== false) {
        logMessage("🔑 Access token refreshed successfully.");
        echo "✅ Token refreshed successfully.<br>";
    } else {
        logMessage("❌ Error refreshing access token. HTTP Code: $httpCode. Response: $response. Error: $error");
        die("❌ Token refresh failed. Exiting...");
    }
}

// ---------- 2. Load Latest Access Token ----------
function loadAccessToken() {
    global $accessToken;

    $conn = dbConnect();
    $result = $conn->query("SELECT access_token FROM emma_mason_api_tokens WHERE client_name='emma_mason' ORDER BY id DESC LIMIT 1");

    if ($result && $result->num_rows > 0) {
        $accessToken = $result->fetch_assoc()['access_token'];
        logMessage("🔓 Access token loaded for fetching reports.");
        echo "🔓 Access token loaded.<br>";
    } else {
        logMessage("❌ Failed to load access token.");
        die("❌ Failed to load access token.");
    }

    $conn->close();
}

// ---------- 3. Fetch and Save Email Report ----------
function fetchAndSaveEmailReport() {
    global $accessToken;
    $reportId = 9;
    $dateFrom = '2023-01-01';
    $dateTo = date('Y-m-d');
    $limit = 100;
    $page = 1;

    echo "🟢 Fetching email report initiated...<br>";
    logMessage("🟢 Fetching email report initiated...");

    while (true) {
        $url = MAUTIC_BASE_URL . "/api/reports/$reportId?dateFrom=$dateFrom&dateTo=$dateTo&limit=$limit&page=$page";
        logMessage("📦 Fetching report from URL: $url");
        echo "📦 Fetching report from URL: $url<br>";

        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, ["Authorization: Bearer $accessToken"]);
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        if ($httpCode === 200) {
            $data = json_decode($response, true);
            $reports = $data['data'] ?? [];

            if (count($reports) > 0) {
                saveEmailReportData($reports);
                $page++;
            } else {
                logMessage("🛑 No more data to fetch.");
                echo "🛑 No more data to fetch.<br>";
                return;
            }
        } else {
            logMessage("❌ Failed to fetch report. HTTP Code: $httpCode");
            echo "❌ Failed to fetch report. HTTP Code: $httpCode<br>";
            return;
        }
    }
}

// ---------- 4. Save Report Data (Insert raw dates into _text fields) ----------
function saveEmailReportData($reports) {
    $conn = dbConnect();

    $stmt = $conn->prepare("REPLACE INTO emma_mason_email_campaign_stats (
        email_id, email_name, subject, category_id, category_title, sent_count, read_count, read_ratio,
        unique_hits, unique_ratio, total_hits, hits_ratio, unsubscribed, unsubscribed_ratio,
        bounced, bounced_ratio, is_published, date_added_text, date_modified_text, publish_up_text, publish_down_text,
        revision, lang
    ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");

    foreach ($reports as $index => $report) {
        logMessage("🔍 Processing report #$index: " . json_encode($report));

        // 🔑 Assign values to variables to pass by reference
        $email_id = (int)($report['e_id'] ?? 0);
        $email_name = $report['e_name'] ?? '';
        $subject = $report['subject1'] ?? '';
        $category_id = !empty($report['category_id']) ? (int)$report['category_id'] : null;
        $category_title = $report['category_title'] ?? '';
        $sent_count = (int)$report['sent_count'];
        $read_count = (int)$report['read_count'];
        $read_ratio = (float)str_replace('%', '', $report['read_ratio']);
        $unique_hits = (int)$report['unique_hits'];
        $unique_ratio = (float)str_replace('%', '', $report['unique_ratio']);
        $total_hits = (int)$report['hits'];
        $hits_ratio = (float)str_replace('%', '', $report['hits_ratio']);
        $unsubscribed = (int)floatval($report['unsubscribed']);
        $unsubscribed_ratio = (float)str_replace('%', '', $report['unsubscribed_ratio']);
        $bounced = (int)floatval($report['bounced']);
        $bounced_ratio = (float)str_replace('%', '', $report['bounced_ratio']);
        $is_published = (int)$report['e_is_published'];

        // ✅ Store raw text dates in _text fields
        $date_added_text = $report['e_date_added'] ?? '';
        $date_modified_text = $report['e_date_modified'] ?? '';
        $publish_up_text = $report['e_publish_up'] ?? '';
        $publish_down_text = $report['e_publish_down'] ?? '';

        $revision = (int)$report['revision'];
        $lang = $report['lang'] ?? '';

        // 🔄 Corrected bind_param with variables
        $stmt->bind_param(
            "issisiidsiidididisssiss",
            $email_id, $email_name, $subject, $category_id, $category_title,
            $sent_count, $read_count, $read_ratio, $unique_hits, $unique_ratio,
            $total_hits, $hits_ratio, $unsubscribed, $unsubscribed_ratio, $bounced,
            $bounced_ratio, $is_published, $date_added_text, $date_modified_text, $publish_up_text,
            $publish_down_text, $revision, $lang
        );

        if ($stmt->execute()) {
            logMessage("✅ Report entry for {$email_name} saved.");
            echo "✅ Report entry for {$email_name} saved.<br>";
        } else {
            logMessage("❌ Error inserting {$email_name}: " . $stmt->error);
        }
    }

    $stmt->close();
    $conn->close();
}

// ---------- 5. Convert Text Dates to DATETIME ----------
function convertTextDatesToDatetime() {
    $conn = dbConnect();

    // Array of all date fields, now including 'publish_down'
    $dateFields = [
        'date_added' => 'date_added_text',
        'date_modified' => 'date_modified_text',
        'publish_up' => 'publish_up_text',
        'publish_down' => 'publish_down_text'  // ✅ Now included
    ];

    foreach ($dateFields as $dateField => $textField) {
        $sql = "
            UPDATE emma_mason_email_campaign_stats
            SET $dateField = STR_TO_DATE($textField, '%Y-%m-%d %H:%i:%s')
            WHERE $textField REGEXP '^[0-9]{4}-[0-9]{2}-[0-9]{2}( [0-9]{2}:[0-9]{2}:[0-9]{2})?$'
        ";

        if ($conn->query($sql) === TRUE) {
            logMessage("✅ $dateField updated successfully.");
            echo "✅ $dateField updated successfully.<br>";
        } else {
            logMessage("❌ Error updating $dateField: " . $conn->error);
            echo "❌ Error updating $dateField: " . $conn->error . "<br>";
        }
    }

    $conn->close();
}


// ---------- 6. Execute All Processes ----------
refreshToken();
loadAccessToken();
fetchAndSaveEmailReport();
convertTextDatesToDatetime();
?>
