<?php
require_once __DIR__ . '/config.php';
require_once __DIR__ . '/includes/utils.php';

define('LOCK_DIR', __DIR__ . '/tmp');
define('LOCK_FILE', LOCK_DIR . '/fetch_contacts.lock');
define('LOCK_TIMEOUT', 30 * 60); // 30 minutes

register_shutdown_function('removeLock');

function createLock() {
    if (!is_dir(LOCK_DIR)) {
        mkdir(LOCK_DIR, 0755, true);
        logMessage("[LOCK] Created lock directory: " . LOCK_DIR);
    }
    if (file_exists(LOCK_FILE)) {
        $fileTime = filemtime(LOCK_FILE);
        if ((time() - $fileTime) < LOCK_TIMEOUT) {
            logMessage("[LOCK] Another instance is running. Exiting...");
            exit;
        }
        unlink(LOCK_FILE); // Remove stale lock
    }
    file_put_contents(LOCK_FILE, getmypid());
    logMessage("[LOCK] Lock acquired.");
}

function resetLeadProcessing() {
    $conn = dbConnect();

    // Check if there are any leads left to process
    $result = $conn->query("SELECT COUNT(*) AS remaining FROM mautic_lead_control WHERE status = 'Not processed'");
    $row = $result->fetch_assoc();
    $remaining = $row['remaining'] ?? 0;

    // If there are no unprocessed leads, reset all to 'Not processed'
    if ($remaining == 0) {
        $conn->query("UPDATE mautic_lead_control SET status = 'Not processed'");
        logMessage("[RESET] No pending leads. Resetting all to 'Not processed'.");
    } else {
        logMessage("[RESET] There are $remaining leads pending. No reset needed.");
    }

    $conn->close();
}


// Run this before fetching leads
createLock();
resetLeadProcessing(); // Reset all leads to 'Not processed'
refreshToken();
loadAccessToken();


function removeLock() {
    if (file_exists(LOCK_FILE)) {
        unlink(LOCK_FILE);
        logMessage("[LOCK] Lock released.");
    }
}

function refreshToken() {
    logMessage("[TOKEN] Refreshing access token...");
    $ch = curl_init('https://www.ssgenius.com/mautic-data-fetcher/token_handler.php');
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    $response = curl_exec($ch);
    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    curl_close($ch);

    if ($httpCode !== 200 || strpos($response, 'Authorization successful') === false) {
        logMessage("[TOKEN] Failed to refresh token.");
        removeLock();
        die("[TOKEN] Token refresh failed.");
    }
}

function loadAccessToken() {
    global $accessToken;
    $conn = dbConnect();
    $result = $conn->query("SELECT access_token FROM emma_mason_api_tokens WHERE client_name='emma_mason' ORDER BY id DESC LIMIT 1");
    if ($result && $result->num_rows > 0) {
        $accessToken = $result->fetch_assoc()['access_token'];
        logMessage("[TOKEN] Access token loaded.");
    } else {
        logMessage("[TOKEN] Failed to load access token.");
        removeLock();
        die("[TOKEN] Failed to load access token.");
    }
    $conn->close();
}

function allProcessed() {
    $conn = dbConnect();
    $result = $conn->query("SELECT COUNT(*) AS remaining FROM mautic_lead_control WHERE status = 'Not processed'");
    $row = $result->fetch_assoc();
    $conn->close();
    return $row['remaining'] == 0;
}


do {
    $contacts = fetchContacts();
    if (empty($contacts)) {
        logMessage("[PROCESS] No contacts fetched in this batch.");
        break;
    }

    logBatchInfo($contacts);
    doInsertsWithMeta($contacts);
} while (!allProcessed());

removeLock();
logMessage("[DONE] Script finished.");
exit;

function fetchContacts() {
    global $accessToken;
    $conn = dbConnect();

    $result = $conn->query("SELECT lead_id FROM mautic_lead_control WHERE status = 'Not processed' LIMIT 100");

    if (!$result || $result->num_rows === 0) {
        logMessage("[FETCH] No unprocessed leads found in mautic_lead_control.");
        return [];
    }

    $leadIds = [];
    while ($row = $result->fetch_assoc()) {
        $leadIds[] = $row['lead_id'];
    }

    $leadIdsString = implode(',', $leadIds);
    $where = [
        [
            'col' => 'id',
            'expr' => 'in',
            'val' => $leadIdsString,
        ]
    ];

    $contacts = [];
    $start = 0;
    $limit = 100; // Adjust this to the maximum allowed by your Mautic instance

    do {
        $query = http_build_query(['where' => $where, 'start' => $start, 'limit' => $limit]);
        $url = MAUTIC_BASE_URL . "/api/contacts?$query";

        logMessage("[FETCH] Fetching contacts from $start to " . ($start + $limit));

        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, ["Authorization: Bearer $accessToken"]);
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        if ($httpCode !== 200) {
            logMessage("[FETCH] Error: HTTP code $httpCode");
            break;
        }

        $data = json_decode($response, true);
        $fetchedContacts = $data['contacts'] ?? [];
        logMessage("[FETCH] Got " . count($fetchedContacts) . " contacts.");

        $contacts = array_merge($contacts, $fetchedContacts);
        $start += $limit;

        if (count($fetchedContacts) < $limit) {
            break;
        }
    } while (true);

    $idsToUpdate = implode(',', array_map('intval', $leadIds));
    $conn->query("UPDATE mautic_lead_control SET status = 'In process' WHERE lead_id IN ($idsToUpdate)");

    $conn->close();
    return $contacts;
}

function logBatchInfo(array $contacts) {
    $logFile = __DIR__ . '/logs/batch_info.log';
    $timestamp = date('Y-m-d H:i:s');
    $message = sprintf("[%s] Processed batch: %d contacts fetched.\n", $timestamp, count($contacts));
    file_put_contents($logFile, $message, FILE_APPEND);
    logMessage($message);
}

function doInsertsWithMeta(array $contacts) {
    $conn = dbConnect();
    $count = 0;
    $errors = 0;

    foreach ($contacts as $contact) {
        $leadId = insertLead($conn, $contact);
        if ($leadId === false) {
            $errors++;
            continue;
        }
        insertMetaFields($conn, $leadId, $contact);
        $count++;
    }

    $logFile = __DIR__ . '/logs/batch_info.log';
    $timestamp = date('Y-m-d H:i:s');
    $message = sprintf("[%s] Inserted: %d contacts, Errors: %d.\n", $timestamp, $count, $errors);
    file_put_contents($logFile, $message, FILE_APPEND);
    logMessage($message);

    $conn->close();
}

function insertLead(mysqli $conn, array $contact) {
    $id = $contact['id'] ?? null;
    $isPublished = $contact['isPublished'] ?? null;
    $points = $contact['points'] ?? null;
    $fields = $contact['fields']['core'] ?? [];

    $email = $fields['email']['value'] ?? null;
    $firstname = $fields['firstname']['value'] ?? null;
    $lastname = $fields['lastname']['value'] ?? null;
    $phone = $fields['phone']['value'] ?? null;
    $city = $fields['city']['value'] ?? null;
    $customerState = $fields['customer_state']['value'] ?? null;
    $zipcode = $fields['zipcode']['value'] ?? null;

    $latestDelOrderDate = fixDateTime($fields['latest_del_order_date']['value'] ?? null);
    $latestDeliveryDate = fixDateTime($fields['latest_delivery_date']['value'] ?? null);
    $firstDelOrderDate = fixDateTime($fields['first_del_order_date']['value'] ?? null);
    $latestOrderDate = fixDateTime($fields['latest_order_date']['value'] ?? null);
    $latestOrderAmount = $fields['latest_order_amount']['value'] ?? null;
    $latestOrderStatus = $fields['latest_order_status']['value'] ?? null;

    $orderProductsList = $fields['order_products_list']['value'] ?? null;
    $orderBrandsList = $fields['order_brands_list']['value'] ?? null;
    $orderProductsCategories = $fields['order_products_categories']['value'] ?? null;
    $orderProductsVendorCol = $fields['order_products_vendor_col']['value'] ?? null;

    // Convert doNotContact array to JSON for storage
    $doNotContact = json_encode($contact['doNotContact'] ?? []);

    // Debugging: Log all values before inserting
    logMessage("[DEBUG] Preparing to insert lead ID=$id");
    logMessage("[DEBUG] isPublished=$isPublished, points=$points, email=$email, firstname=$firstname, lastname=$lastname");
    logMessage("[DEBUG] phone=$phone, city=$city, customerState=$customerState, zipcode=$zipcode");
    logMessage("[DEBUG] latestOrderAmount=$latestOrderAmount, latestOrderStatus=$latestOrderStatus, doNotContact=$doNotContact");

    $sql = "
        INSERT INTO mautic_leads (
            id, is_published, points, email, firstname, lastname, phone, city, customer_state, zipcode,
            latest_del_order_date, latest_delivery_date, first_del_order_date,
            latest_order_date, latest_order_amount, latest_order_status,
            order_products_list, order_brands_list, order_products_categories, order_products_vendor_col, do_not_contact
        ) VALUES (
            ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?
        )
        ON DUPLICATE KEY UPDATE
            is_published=VALUES(is_published), points=VALUES(points), email=VALUES(email), 
            firstname=VALUES(firstname), lastname=VALUES(lastname), phone=VALUES(phone),
            city=VALUES(city), customer_state=VALUES(customer_state), zipcode=VALUES(zipcode),
            latest_del_order_date=VALUES(latest_del_order_date),
            latest_delivery_date=VALUES(latest_delivery_date),
            first_del_order_date=VALUES(first_del_order_date),
            latest_order_date=VALUES(latest_order_date),
            latest_order_amount=VALUES(latest_order_amount),
            latest_order_status=VALUES(latest_order_status),
            order_products_list=VALUES(order_products_list),
            order_brands_list=VALUES(order_brands_list),
            order_products_categories=VALUES(order_products_categories),
            order_products_vendor_col=VALUES(order_products_vendor_col),
            do_not_contact=VALUES(do_not_contact)
    ";

    $stmt = $conn->prepare($sql);
    if (!$stmt) {
        logMessage("[DB] Prepare failed: " . $conn->error);
        return false;
    }

    $bindTypes = 'iisssssssssssssssssss';
    $stmt->bind_param($bindTypes, $id, $isPublished, $points, $email, $firstname, $lastname, $phone, $city, $customerState, $zipcode,
        $latestDelOrderDate, $latestDeliveryDate, $firstDelOrderDate, $latestOrderDate,
        $latestOrderAmount, $latestOrderStatus, $orderProductsList, $orderBrandsList, 
        $orderProductsCategories, $orderProductsVendorCol, $doNotContact);

    if (!$stmt->execute()) {
        logMessage("[DB] Insert failed ID=$id: " . $stmt->error);
        $stmt->close();
        return false;
    }

    logMessage("[DB] Inserted/Updated lead ID=$id in mautic_leads");
    $stmt->close();
    return $id;
}


function insertMetaFields(mysqli $conn, $leadId, array $contact) {
    // Handle JSON fields like ipAddresses, tags, utmtags

    if (!empty($contact['ipAddresses'])) {
        $rawJson = json_encode($contact['ipAddresses']);
        insertMeta($conn, $leadId, 'ip_addresses', $rawJson);
    }

    if (!empty($contact['tags'])) {
        $rawJson = json_encode($contact['tags']);
        insertMeta($conn, $leadId, 'tags', $rawJson);
    }

    if (!empty($contact['utmtags'])) {
        $rawJson = json_encode($contact['utmtags']);
        insertMeta($conn, $leadId, 'utm_tags', $rawJson);
    }
}

function insertMeta(mysqli $conn, int $leadId, string $metaKey, string $metaValue) {
    $sql = "
        INSERT INTO mautic_leads_meta (lead_id, meta_key, meta_value)
        VALUES (?, ?, ?)
        ON DUPLICATE KEY UPDATE meta_value=VALUES(meta_value)
    ";

    $stmt = $conn->prepare($sql);
    if (!$stmt) {
        logMessage("[DB] insertMeta prepare failed: " . $conn->error);
        return;
    }

    $stmt->bind_param("iss", $leadId, $metaKey, $metaValue);
    if (!$stmt->execute()) {
        logMessage("[DB] insertMeta failed (lead_id=$leadId, key=$metaKey): " . $stmt->error);
    } else {
        logMessage("[DB] meta inserted => $leadId, $metaKey");
    }
    $stmt->close();
}

function fixDateTime($raw) {
    if (empty($raw)) {
        return null;
    }
    try {
        $dt = new DateTime($raw);
        return $dt->format('Y-m-d H:i:s');
    } catch (Exception $e) {
        return null;
    }
}
