<?php
/**
 * populate_email_sales_funnel_report.php
 * 
 * This script populates the following reports:
 * - State Sales Report
 * - Brand Sales Report
 * - Collection Sales Report
 */

require_once __DIR__ . '/../config.php';
require_once __DIR__ . '/utils.php';

// -----------------------------------------------
// Function to print logs in HTML
// -----------------------------------------------
function logCustomMessage($message) {
    echo "<p>" . htmlspecialchars($message) . "</p>\n";
}

// -----------------------------------------------
// Database Connection
// -----------------------------------------------
$conn = new mysqli(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);
if ($conn->connect_error) {
    die("<p style='color:red;'>❌ Database connection error: " . $conn->connect_error . "</p>");
}
$conn->set_charset("utf8mb4");

// -----------------------------------------------
// Function: Populate State Sales Report
// -----------------------------------------------
function populateStateSalesReport($conn) {
    logCustomMessage("<strong>Step 1:</strong> Populating `email_sales_funnel_state_sales_report`");

    // Clear existing data
    $sqlTruncate = "TRUNCATE TABLE email_sales_funnel_state_sales_report";
    if (!$conn->query($sqlTruncate)) {
        logCustomMessage("❌ Error truncating state sales report: " . $conn->error);
        return;
    }

    // Insert new data
    $sqlInsert = "
        INSERT INTO email_sales_funnel_state_sales_report (state, total_contacts, total_order_value)
        SELECT 
            CASE 
                WHEN customer_state IN (
                    'Alabama', 'Alaska', 'Arizona', 'Arkansas', 'California', 'Colorado', 'Connecticut', 'Delaware', 
                    'Florida', 'Georgia', 'Hawaii', 'Idaho', 'Illinois', 'Indiana', 'Iowa', 'Kansas', 'Kentucky', 
                    'Louisiana', 'Maine', 'Maryland', 'Massachusetts', 'Michigan', 'Minnesota', 'Mississippi', 
                    'Missouri', 'Montana', 'Nebraska', 'Nevada', 'New Hampshire', 'New Jersey', 'New Mexico', 
                    'New York', 'North Carolina', 'North Dakota', 'Ohio', 'Oklahoma', 'Oregon', 'Pennsylvania', 
                    'Rhode Island', 'South Carolina', 'South Dakota', 'Tennessee', 'Texas', 'Utah', 'Vermont', 
                    'Virginia', 'Washington', 'West Virginia', 'Wisconsin', 'Wyoming'
                ) THEN customer_state
                ELSE 'Not buyer'
            END AS state,
            COUNT(*) AS total_contacts,
            COALESCE(SUM(latest_order_amount), 0) AS total_order_value
        FROM mautic_leads
        GROUP BY state
        ORDER BY total_contacts DESC;
    ";

    if ($conn->query($sqlInsert)) {
        logCustomMessage("✅ State Sales Report populated successfully.");
    } else {
        logCustomMessage("❌ Error inserting into state sales report: " . $conn->error);
    }
}

// -----------------------------------------------
// Function: Populate Brand Sales Report
// -----------------------------------------------
function populateBrandSalesReport($conn) {
    logCustomMessage("<strong>Step 2:</strong> Populating `email_sales_funnel_brand_sales_report`");

    // Clear existing data
    $sqlTruncate = "TRUNCATE TABLE email_sales_funnel_brand_sales_report";
    if (!$conn->query($sqlTruncate)) {
        logCustomMessage("❌ Error truncating brand sales report: " . $conn->error);
        return;
    }

    // Insert new data
    $sqlInsert = "
        INSERT INTO email_sales_funnel_brand_sales_report (brand, total_repetitions, total_sales)
        SELECT 
            brand,
            COUNT(*) AS total_repetitions,
            COALESCE(SUM(latest_order_amount), 0) AS total_sales
        FROM (
            SELECT 
                TRIM(value) AS brand, 
                latest_order_amount
            FROM mautic_leads
            JOIN JSON_TABLE(
                CONCAT('[\"', REPLACE(order_brands_list, ',', '\",\"'), '\"]'), 
                '$[*]' COLUMNS (value VARCHAR(255) PATH '$')
            ) AS brand_table
        ) AS extracted_brands
        GROUP BY brand
        ORDER BY total_repetitions DESC, total_sales DESC;
    ";

    if ($conn->query($sqlInsert)) {
        logCustomMessage("✅ Brand Sales Report populated successfully.");
    } else {
        logCustomMessage("❌ Error inserting into brand sales report: " . $conn->error);
    }
}


// -----------------------------------------------
// Function: Populate Collection & Brand Sales Report
// -----------------------------------------------
function populateCollectionSalesReport($conn)
{
    logCustomMessage("<strong>Step 3:</strong> Populating `email_sales_funnel_collection_sales_report`");

    // 1) Truncate table
    $sqlTruncate = "TRUNCATE TABLE email_sales_funnel_collection_sales_report";
    if (!$conn->query($sqlTruncate)) {
        logCustomMessage("❌ Error truncating collection sales report: " . $conn->error);
        return;
    }

    // 2) Insert DISTINCT (collection, brand) combos
    //    using CROSS JOIN with numbers table, forcing COLLATE utf8mb4_unicode_ci
    $sqlInsertDistinctCombos = "
        INSERT INTO email_sales_funnel_collection_sales_report (collection, brand)
        SELECT DISTINCT
            TRIM(
                SUBSTRING_INDEX(
                    SUBSTRING_INDEX(
                        ml.order_products_vendor_col COLLATE utf8mb4_unicode_ci,
                        ',',
                        col_numbers.n
                    ),
                    ',',
                    -1
                )
            ) COLLATE utf8mb4_unicode_ci AS collection,
            
            TRIM(
                SUBSTRING_INDEX(
                    SUBSTRING_INDEX(
                        ml.order_brands_list COLLATE utf8mb4_unicode_ci,
                        ',',
                        brand_numbers.n
                    ),
                    ',',
                    -1
                )
            ) COLLATE utf8mb4_unicode_ci AS brand
        FROM mautic_leads AS ml
        
        -- Numbers table for collections
        CROSS JOIN (
            SELECT 1 AS n UNION ALL SELECT 2 UNION ALL SELECT 3 
            UNION ALL SELECT 4 UNION ALL SELECT 5
            UNION ALL SELECT 6 UNION ALL SELECT 7 
            UNION ALL SELECT 8 UNION ALL SELECT 9 
            UNION ALL SELECT 10
        ) AS col_numbers
        
        -- Numbers table for brands
        CROSS JOIN (
            SELECT 1 AS n UNION ALL SELECT 2 UNION ALL SELECT 3 
            UNION ALL SELECT 4 UNION ALL SELECT 5
            UNION ALL SELECT 6 UNION ALL SELECT 7 
            UNION ALL SELECT 8 UNION ALL SELECT 9 
            UNION ALL SELECT 10
        ) AS brand_numbers
        
        WHERE
            -- Only expand as many commas as exist
            CHAR_LENGTH(ml.order_products_vendor_col) 
             - CHAR_LENGTH(REPLACE(ml.order_products_vendor_col, ',', '')) >= col_numbers.n - 1
            AND
            CHAR_LENGTH(ml.order_brands_list) 
             - CHAR_LENGTH(REPLACE(ml.order_brands_list, ',', '')) >= brand_numbers.n - 1
            
            -- Avoid empty strings
            AND TRIM(
                SUBSTRING_INDEX(
                    SUBSTRING_INDEX(
                        ml.order_products_vendor_col COLLATE utf8mb4_unicode_ci,
                        ',',
                        col_numbers.n
                    ),
                    ',',
                    -1
                )
            ) <> ''
            AND TRIM(
                SUBSTRING_INDEX(
                    SUBSTRING_INDEX(
                        ml.order_brands_list COLLATE utf8mb4_unicode_ci,
                        ',',
                        brand_numbers.n
                    ),
                    ',',
                    -1
                )
            ) <> ''
    ";

    if (!$conn->query($sqlInsertDistinctCombos)) {
        logCustomMessage("❌ Error inserting distinct combos: " . $conn->error);
        return;
    } else {
        logCustomMessage("✅ Distinct (collection, brand) combos inserted successfully.");
    }

    // 3) Update totals (total_repetitions, total_sales) by JOINing on (collection, brand),
    //    again forcing COLLATE utf8mb4_unicode_ci
    $sqlUpdateTotals = "
        UPDATE email_sales_funnel_collection_sales_report AS csr
        JOIN (
            SELECT
                TRIM(
                    SUBSTRING_INDEX(
                        SUBSTRING_INDEX(
                            ml.order_products_vendor_col COLLATE utf8mb4_unicode_ci,
                            ',',
                            col_numbers.n
                        ),
                        ',',
                        -1
                    )
                ) COLLATE utf8mb4_unicode_ci AS collection,
                
                TRIM(
                    SUBSTRING_INDEX(
                        SUBSTRING_INDEX(
                            ml.order_brands_list COLLATE utf8mb4_unicode_ci,
                            ',',
                            brand_numbers.n
                        ),
                        ',',
                        -1
                    )
                ) COLLATE utf8mb4_unicode_ci AS brand,
                
                COUNT(*) AS count_repetitions,
                COALESCE(SUM(ml.latest_order_amount), 0) AS sum_sales
            FROM mautic_leads AS ml
            CROSS JOIN (
                SELECT 1 AS n UNION ALL SELECT 2 UNION ALL SELECT 3 
                UNION ALL SELECT 4 UNION ALL SELECT 5
                UNION ALL SELECT 6 UNION ALL SELECT 7 
                UNION ALL SELECT 8 UNION ALL SELECT 9 
                UNION ALL SELECT 10
            ) AS col_numbers
            CROSS JOIN (
                SELECT 1 AS n UNION ALL SELECT 2 UNION ALL SELECT 3 
                UNION ALL SELECT 4 UNION ALL SELECT 5
                UNION ALL SELECT 6 UNION ALL SELECT 7 
                UNION ALL SELECT 8 UNION ALL SELECT 9 
                UNION ALL SELECT 10
            ) AS brand_numbers
            WHERE
                CHAR_LENGTH(ml.order_products_vendor_col) 
                 - CHAR_LENGTH(REPLACE(ml.order_products_vendor_col, ',', '')) >= col_numbers.n - 1
                AND
                CHAR_LENGTH(ml.order_brands_list) 
                 - CHAR_LENGTH(REPLACE(ml.order_brands_list, ',', '')) >= brand_numbers.n - 1
                AND TRIM(
                    SUBSTRING_INDEX(
                        SUBSTRING_INDEX(
                            ml.order_products_vendor_col COLLATE utf8mb4_unicode_ci,
                            ',',
                            col_numbers.n
                        ),
                        ',',
                        -1
                    )
                ) <> ''
                AND TRIM(
                    SUBSTRING_INDEX(
                        SUBSTRING_INDEX(
                            ml.order_brands_list COLLATE utf8mb4_unicode_ci,
                            ',',
                            brand_numbers.n
                        ),
                        ',',
                        -1
                    )
                ) <> ''
            GROUP BY collection, brand
        ) AS sub
            ON csr.collection COLLATE utf8mb4_unicode_ci = sub.collection
           AND csr.brand COLLATE utf8mb4_unicode_ci = sub.brand
        SET csr.total_repetitions = sub.count_repetitions,
            csr.total_sales       = sub.sum_sales
    ";

    if (!$conn->query($sqlUpdateTotals)) {
        logCustomMessage("❌ Error updating totals: " . $conn->error);
        return;
    } else {
        logCustomMessage("✅ Totals updated successfully in `email_sales_funnel_collection_sales_report`.");
    }

    logCustomMessage("✅ Collection & Brand Sales Report completed.");
}

// -----------------------------------------------
// Function: Populate State & Brand Sales Report
// -----------------------------------------------
function populateStateBrandCollectionSalesReport($conn)
{
    logCustomMessage("<strong>Populating</strong> `email_sales_funnel_state_brand_sales_report` with ALL combos (including empty => 'N/A')");

    // 1) Truncate table
    $sqlTruncate = "TRUNCATE TABLE email_sales_funnel_state_brand_sales_report";
    if (!$conn->query($sqlTruncate)) {
        logCustomMessage("❌ Error truncating: " . $conn->error);
        return;
    }

    // 2) Single-step INSERT
    //    Build all (state, brand, collection), fallback to 'N/A' if empty
    $sqlInsert = "
        INSERT INTO email_sales_funnel_state_brand_sales_report
            (state, brand, collection, total_repetitions, total_sales)
        SELECT
            -- Fallback to 'N/A' if state is empty
            COALESCE(
                NULLIF(
                    TRIM(ml.customer_state COLLATE utf8mb4_unicode_ci),
                    ''
                ),
                'N/A'
            ) AS state,

            -- Brand expansions (up to 10 items),
            -- fallback to 'N/A' if empty
            COALESCE(
                NULLIF(
                    TRIM(
                        SUBSTRING_INDEX(
                            SUBSTRING_INDEX(
                                ml.order_brands_list COLLATE utf8mb4_unicode_ci,
                                ',',
                                brand_numbers.n
                            ),
                            ',',
                            -1
                        )
                    ),
                    ''
                ),
                'N/A'
            ) AS brand,

            -- Collection expansions (up to 10 items),
            -- fallback to 'N/A' if empty
            COALESCE(
                NULLIF(
                    TRIM(
                        SUBSTRING_INDEX(
                            SUBSTRING_INDEX(
                                ml.order_products_vendor_col COLLATE utf8mb4_unicode_ci,
                                ',',
                                col_numbers.n
                            ),
                            ',',
                            -1
                        )
                    ),
                    ''
                ),
                'N/A'
            ) AS collection,

            COUNT(*) AS total_repetitions,
            COALESCE(SUM(ml.latest_order_amount), 0) AS total_sales
        FROM mautic_leads ml

        -- Expand brand up to 10 slices
        CROSS JOIN (
            SELECT 1 AS n UNION ALL SELECT 2 UNION ALL SELECT 3
            UNION ALL SELECT 4 UNION ALL SELECT 5
            UNION ALL SELECT 6 UNION ALL SELECT 7
            UNION ALL SELECT 8 UNION ALL SELECT 9
            UNION ALL SELECT 10
        ) AS brand_numbers

        -- Expand collection up to 10 slices
        CROSS JOIN (
            SELECT 1 AS n UNION ALL SELECT 2 UNION ALL SELECT 3
            UNION ALL SELECT 4 UNION ALL SELECT 5
            UNION ALL SELECT 6 UNION ALL SELECT 7
            UNION ALL SELECT 8 UNION ALL SELECT 9
            UNION ALL SELECT 10
        ) AS col_numbers

        GROUP BY state, brand, collection
        ORDER BY total_repetitions DESC, total_sales DESC
    ";

    if ($conn->query($sqlInsert)) {
        logCustomMessage("✅ ALL (state, brand, collection) combos inserted successfully!");
    } else {
        logCustomMessage("❌ Error inserting combos: " . $conn->error);
        return;
    }
}





// -----------------------------------------------
// **Execute Functions to Populate Reports**
// -----------------------------------------------
populateStateSalesReport($conn);
populateBrandSalesReport($conn);
populateCollectionSalesReport($conn);
populateStateBrandCollectionSalesReport($conn);

// -----------------------------------------------
// Close Database Connection
// -----------------------------------------------
$conn->close();
logCustomMessage("✅ <strong>Process completed successfully.</strong>");
?>
