<?php
/**
 * Populates the email_sales_funnel_report_daily table with daily aggregated sales funnel data.
 */

require_once __DIR__ . '/../config.php';
require_once __DIR__ . '/utils.php';

// -----------------------------------------------
// Database Connection
// -----------------------------------------------
$conn = new mysqli(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);
if ($conn->connect_error) {
    die("<p style='color:red;'>❌ Database connection error: " . $conn->connect_error . "</p>");
}
$conn->set_charset("utf8mb4");

// Function to log messages
function logCustomMessage($message) {
    echo "<p>" . htmlspecialchars($message) . "</p>\n";
}

// -----------------------------------------------
// Step 1: Truncate the daily report table
// -----------------------------------------------
function truncateDailyReport($conn) {
    logCustomMessage("<strong>Step 1:</strong> Truncating email_sales_funnel_report_daily");
    $sql = "TRUNCATE TABLE email_sales_funnel_report_daily";
    return $conn->query($sql)
        ? logCustomMessage("✅ Table truncated successfully.") 
        : logCustomMessage("❌ SQL Error truncating table: " . $conn->error);
}

// -----------------------------------------------
// Step 2: Insert All Possible Rows
//  (Filter out NULL email_id to avoid insertion errors)
// -----------------------------------------------
function insertDailyFunnelRows($conn) {
    logCustomMessage("<strong>Step 2:</strong> Creating all possible rows for email_sales_funnel_report_daily");

    $sql = "
    INSERT INTO email_sales_funnel_report_daily
        (email_id, calendar_date, latest_order_status, total_sent, total_opens, total_clicks, status_stage_type)
    SELECT 
        emails.email_id,
        dates.calendar_date,
        statuses.latest_order_status,
        0 AS total_sent,
        0 AS total_opens,
        0 AS total_clicks,
        NULL AS status_stage_type
    FROM 
        (
            SELECT DISTINCT email_id 
            FROM email_activity_per_day
            WHERE email_id IS NOT NULL        /* ✅ Filter out NULL email_id */
        ) AS emails
    CROSS JOIN 
        (
            SELECT DISTINCT calendar_date
            FROM email_activity_per_day
        ) AS dates
    CROSS JOIN 
        (
            SELECT DISTINCT latest_order_status
            FROM email_sales_funnel_report
        ) AS statuses
    ";

    if ($conn->query($sql)) {
        logCustomMessage("✅ Rows inserted successfully into email_sales_funnel_report_daily.");
    } else {
        logCustomMessage("❌ SQL Error inserting rows: " . $conn->error);
    }
}

// -----------------------------------------------
// Step 3: Populate Basic Daily Values (No Calculations Needed)
// -----------------------------------------------
function populateBasicDailyFunnelValues($conn) {
    logCustomMessage("<strong>Step 3:</strong> Populating basic values in email_sales_funnel_report_daily");

    $updateQueries = [
        "total_sent" => "
            UPDATE email_sales_funnel_report_daily d
            JOIN email_activity_per_day a 
               ON d.email_id = a.email_id 
              AND d.calendar_date = a.calendar_date
            SET d.total_sent = a.total_sent;
        ",

        "total_opens" => "
            UPDATE email_sales_funnel_report_daily d
            JOIN email_activity_per_day a 
               ON d.email_id = a.email_id
              AND d.calendar_date = a.calendar_date
            SET d.total_opens = a.total_opens;
        ",

        "total_clicks" => "
            UPDATE email_sales_funnel_report_daily d
            JOIN email_activity_per_day a 
               ON d.email_id = a.email_id
              AND d.calendar_date = a.calendar_date
            SET d.total_clicks = a.total_clicks;
        "
    ];

    foreach ($updateQueries as $stage => $query) {
        if ($conn->query($query)) {
            logCustomMessage("✅ Successfully updated $stage.");
        } else {
            logCustomMessage("❌ SQL Error updating $stage: " . $conn->error);
        }
    }
}

// -----------------------------------------------
// Step 4: Populate status_stage_type
// -----------------------------------------------
function updateStatusStageType($conn) {
    logCustomMessage("<strong>Step 4:</strong> Updating status_stage_type in email_sales_funnel_report_daily");

    $sql = "
    UPDATE email_sales_funnel_report_daily
    SET status_stage_type = 
        CASE 
            WHEN latest_order_status IN (
                'Delivered', 'Ready for Pickup', 'Shipped', 'At Hub'
            ) THEN 'Completed or Successfully Processed Orders'

            WHEN latest_order_status IN (
                'Ordered', 'Processing', 'Scheduled', 
                'Shipped, Need Payment', 'Scheduled, Need Payment'
            ) THEN 'Orders In Process but Likely Paid'

            WHEN latest_order_status IN (
                'Cancelled','Need To Cancel','Chargeback','Delivered - Open Issues',
                'Cust. Has Balance Due','Delivered, Need Payment','Customer Issue',
                'Syn-delivered open issue','Backordered','Syn-Backordered','Pending Inventory',
                'Reauth. Required','Pending Cust. Info','Order On Hold','Pending Check Clear'
            ) THEN 'Uncertain, Pending, or Cancelled Orders'

            ELSE 'Unknown'
        END
    ";

    if ($conn->query($sql)) {
        logCustomMessage("✅ Successfully updated status_stage_type.");
    } else {
        logCustomMessage("❌ SQL Error updating status_stage_type: " . $conn->error);
    }
}

// -----------------------------------------------
// Step 5: Populate total_bounces
// -----------------------------------------------
function populateBounces($conn) {
    logCustomMessage("<strong>Step 5:</strong> Populating total_bounces in email_sales_funnel_report_daily");

    $sql = "
    UPDATE email_sales_funnel_report_daily d
    JOIN (
        SELECT 
            mbe.email_id, 
            DATE(mbe.created_at) AS calendar_date, 
            COALESCE(ml.latest_order_status, 'Unknown') AS latest_order_status, 
            COUNT(*) AS total_bounces
        FROM mautic_bounce_by_webhook_events mbe
        JOIN mautic_leads ml ON mbe.lead_id = ml.id
        WHERE mbe.bounce_id IS NOT NULL  -- Ensure only valid bounces
        GROUP BY mbe.email_id, DATE(mbe.created_at), ml.latest_order_status
    ) AS bounce_data
       ON d.email_id = bounce_data.email_id
      AND d.calendar_date = bounce_data.calendar_date
      AND d.latest_order_status = bounce_data.latest_order_status
    SET d.total_bounces = bounce_data.total_bounces;
    ";

    if ($conn->query($sql)) {
        logCustomMessage("✅ Successfully updated total_bounces.");
    } else {
        logCustomMessage("❌ SQL Error updating total_bounces: " . $conn->error);
    }
}

// -----------------------------------------------
// Step 6: Populate total_unique_clickers
// -----------------------------------------------
function populateUniqueClickers($conn) {
    logCustomMessage("<strong>Step 6:</strong> Populating total_unique_clickers in email_sales_funnel_report_daily");

    $sql = "
    UPDATE email_sales_funnel_report_daily d
    JOIN (
        SELECT 
            pcd.email_id, 
            DATE(pcd.click_date) AS calendar_date, 
            COALESCE(ml.latest_order_status, 'No Order') AS latest_order_status, 
            COUNT(DISTINCT pcd.lead_id) AS total_unique_clickers
        FROM parsed_click_data pcd
        LEFT JOIN mautic_leads ml ON pcd.lead_id = ml.id
        WHERE pcd.click_date IS NOT NULL
        GROUP BY pcd.email_id, DATE(pcd.click_date), ml.latest_order_status
    ) clickers_data
    ON d.email_id = clickers_data.email_id 
    AND d.calendar_date = clickers_data.calendar_date 
    AND d.latest_order_status = clickers_data.latest_order_status
    SET d.total_unique_clickers = clickers_data.total_unique_clickers;
    ";

    return $conn->query($sql) ? logCustomMessage("✅ Successfully updated total_unique_clickers.") 
                              : logCustomMessage("❌ SQL Error updating total_unique_clickers: " . $conn->error);
}


// -----------------------------------------------
// Step 7: Populate Remaining Indicators (To be implemented later)
// -----------------------------------------------
/*
function populateRemainingIndicators($conn) {
    logCustomMessage("<strong>Step 7:</strong> Populating remaining indicators (Not implemented yet)");
}
*/

/**
 * Step 7.1 - Part 1: Poblar la tabla staging_sales_data con datos agregados de ventas
 */
function populateStagingSalesData($conn) {
    echo "<strong>Step 7.1 (Part 1):</strong> Populating staging table...<br>";

    // 1. **Vaciar la tabla de staging**
    if (!$conn->query("TRUNCATE TABLE staging_sales_data")) {
        echo "❌ Error truncating staging table: " . $conn->error . "<br>";
        return false;
    }

    // 2. **Insertar datos en la tabla de staging**
    $sqlInsert = "
        INSERT INTO staging_sales_data (email_id, send_date, latest_order_status, total_sales, total_transactions)
        SELECT
            msf.email_id,
            DATE(msf.date_sent) AS send_date,
            ml.latest_order_status,
            SUM(ml.latest_order_amount) AS total_sales,
            COUNT(*) AS total_transactions
        FROM mautic_stats_email_full AS msf
        JOIN mautic_leads AS ml ON msf.lead_id = ml.id
        WHERE msf.is_failed = 0  
          AND ml.latest_order_status IS NOT NULL
          AND ml.latest_order_amount IS NOT NULL
        GROUP BY msf.email_id, DATE(msf.date_sent), ml.latest_order_status
    ";

    if (!$conn->query($sqlInsert)) {
        echo "❌ Error inserting into staging table: " . $conn->error . "<br>";
        return false;
    }

    echo "✅ Staging table populated successfully.<br>";
    return true;
}



/**
 * Step 7.2 - Poblar la tabla staging_sales_data con datos desde send_date en adelante

function populateStagingSalesDataAfterSendDate($conn) {
    logCustomMessage("<strong>Step 7.2:</strong> Populating staging table with post-send_date data in batches...");

    // 1. Obtener todos los email_id pendientes de procesamiento
    $query = "SELECT DISTINCT email_id FROM staging_sales_data WHERE after_send_date_process_status = 'pending' ORDER BY email_id ASC";
    $result = $conn->query($query);
    
    $processed_emails = 0;
    while ($row = $result->fetch_assoc()) {
        $processing_email_id = $row['email_id'];
        logCustomMessage("🔍 Processing email_id: " . $processing_email_id);
        
        // Obtener todas las fechas pendientes para este email_id
        $dateQuery = "SELECT DISTINCT send_date FROM staging_sales_data WHERE email_id = ? AND after_send_date_process_status = 'pending' ORDER BY send_date ASC";
        $dateStmt = $conn->prepare($dateQuery);
        $dateStmt->bind_param("i", $processing_email_id);
        $dateStmt->execute();
        $dateResult = $dateStmt->get_result();
        
        while ($dateRow = $dateResult->fetch_assoc()) {
            $processing_date = $dateRow['send_date'];
            logCustomMessage("🔍 Processing send_date: " . $processing_date . " for email_id: " . $processing_email_id);

            // Poblar los valores de ventas y transacciones solo desde send_date en adelante
            $sqlUpdate = "
                UPDATE staging_sales_data AS s
                JOIN (
                    SELECT 
                        msf.email_id,
                        ml.id AS lead_id,
                        DATE(msf.date_sent) AS send_date,
                        ml.latest_order_status,
                        SUM(ml.latest_order_amount) AS total_sales_after_send_date,
                        COUNT(*) AS total_transactions_after_send_date
                    FROM mautic_stats_email_full AS msf
                    JOIN mautic_leads AS ml ON msf.lead_id = ml.id
                    WHERE msf.is_failed = 0  
                      AND ml.latest_order_status IS NOT NULL
                      AND ml.latest_order_amount IS NOT NULL
                      AND msf.date_sent <= ml.latest_order_date -- Asegurar que la venta ocurrió después del email enviado
                      AND msf.date_sent >= ? -- Solo desde send_date en adelante
                      AND msf.email_id = ? -- Procesar solo el email_id actual
                    GROUP BY msf.email_id, ml.id, DATE(msf.date_sent), ml.latest_order_status
                ) AS new_data
                ON s.email_id = new_data.email_id AND s.send_date = new_data.send_date AND s.latest_order_status = new_data.latest_order_status
                SET 
                    s.total_sales_after_send_date = new_data.total_sales_after_send_date,
                    s.total_transactions_after_send_date = new_data.total_transactions_after_send_date,
                    s.after_send_date_processed_at = NOW(),
                    s.after_send_date_process_status = 'completed';
            ";

            $stmt = $conn->prepare($sqlUpdate);
            $stmt->bind_param("si", $processing_date, $processing_email_id);
            if ($stmt->execute()) {
                logCustomMessage("✅ Successfully updated staging_sales_data for send_date: " . $processing_date . " and email_id: " . $processing_email_id);
                logCustomMessage("📊 Rows affected: " . $stmt->affected_rows);
            } else {
                logCustomMessage("❌ SQL Error updating staging_sales_data: " . $stmt->error);
            }
            $stmt->close();
        }
        
        $dateStmt->close();
        $processed_emails++;
    }
    
    logCustomMessage("✅ Total email_ids processed: " . $processed_emails);
}
 */



// -----------------------------------------------
// Execute All Steps
// -----------------------------------------------
truncateDailyReport($conn);
insertDailyFunnelRows($conn);
populateBasicDailyFunnelValues($conn);
updateStatusStageType($conn);
populateBounces($conn);
populateUniqueClickers($conn);
populateStagingSalesData($conn);
//populateStagingSalesDataAfterSendDate($conn);



// Close connection
$conn->close();
logCustomMessage("✅ <strong>Process completed successfully.</strong>");
?>
