<?php
/**
 * Populates the initial structure of email_sales_performance_tracking_over_time_report.
 * This script pulls email IDs and names from emma_mason_email_details
 * and ensures that 19 rows per email exist (16 order statuses + 3 status groups).
 * Existing records are updated instead of duplicated.
 * Additionally, it fetches the first_send_date from mautic_stats_email_full.
 */

require_once __DIR__ . '/../config.php';
require_once __DIR__ . '/utils.php';

// -----------------------------------------------
// Database Connection
// -----------------------------------------------
$conn = new mysqli(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);
if ($conn->connect_error) {
    die("<p style='color:red;'>❌ Database connection error: " . $conn->connect_error . "</p>");
}
$conn->set_charset("utf8mb4");

// Function to log messages
function logCustomMessage($message) {
    echo "<p>" . htmlspecialchars($message) . "</p>\n";
}

// -----------------------------------------------
// Step 1: Get email_id and email_name from emma_mason_email_details
// -----------------------------------------------
function fetchEmailDetails($conn) {
    logCustomMessage("<strong>Step 1:</strong> Fetching email details");
    
    $query = "SELECT email_id, name FROM emma_mason_email_details";
    $result = $conn->query($query);
    
    $emails = [];
    while ($row = $result->fetch_assoc()) {
        $emails[] = $row;
    }
    return $emails;
}

// -----------------------------------------------
// Step 2: Fetch unique latest_order_status values from mautic_leads
// -----------------------------------------------
function fetchUniqueOrderStatuses($conn) {
    logCustomMessage("<strong>Step 2:</strong> Fetching unique latest_order_status values");
    
    $query = "SELECT DISTINCT latest_order_status FROM mautic_leads WHERE latest_order_status IS NOT NULL";
    $result = $conn->query($query);
    
    $statuses = [];
    while ($row = $result->fetch_assoc()) {
        $statuses[] = $row['latest_order_status'] ?: 'Unknown';
    }
    
    // Add the 3 consolidated status groups
    $statuses[] = "Completed or Successfully Processed Orders";
    $statuses[] = "Orders In Process but Likely Paid";
    $statuses[] = "Uncertain, Pending, or Cancelled Orders";
    
    return $statuses;
}

// -----------------------------------------------
// Step 3: Populate or Update the report table structure
// -----------------------------------------------
function populateOrUpdateReportStructure($conn) {
    $emails = fetchEmailDetails($conn);
    $statuses = fetchUniqueOrderStatuses($conn);
    
    logCustomMessage("<strong>Step 3:</strong> Populating or updating report table structure");
    
    foreach ($emails as $email) {
        foreach ($statuses as $status) {
            $sql = "INSERT INTO email_sales_performance_tracking_over_time_report (email_id, email_name, latest_order_status) 
                    SELECT ?, ?, ? FROM DUAL 
                    WHERE NOT EXISTS (
                        SELECT 1 FROM email_sales_performance_tracking_over_time_report
                        WHERE email_id = ? AND latest_order_status = ?
                    )";
            
            $stmt = $conn->prepare($sql);
            $stmt->bind_param("issss", $email['email_id'], $email['name'], $status, $email['email_id'], $status);
            $stmt->execute();
            $stmt->close();
        }
    }
    
    logCustomMessage("✅ Report structure populated or updated successfully.");
}

// -----------------------------------------------
// Step 4: Update first_send_date using mautic_stats_email_full
// -----------------------------------------------
function updateFirstSendDate($conn) {
    logCustomMessage("<strong>Step 4:</strong> Updating first_send_date");
    
    $sql = "
        UPDATE email_sales_performance_tracking_over_time_report e
        JOIN (
            SELECT email_id, MIN(date_sent) AS first_send_date
            FROM mautic_stats_email_full
            WHERE email_id IS NOT NULL
            GROUP BY email_id
        ) msf ON e.email_id = msf.email_id
        SET e.first_send_date = msf.first_send_date;
    ";
    
    if ($conn->query($sql)) {
        logCustomMessage("✅ Successfully updated first_send_date.");
    } else {
        logCustomMessage("❌ SQL Error updating first_send_date: " . $conn->error);
    }
}

// -----------------------------------------------
// Step 5: Update campaign_id using emma_mason_campaign_events
// -----------------------------------------------
function updateCampaignId($conn) {
    logCustomMessage("<strong>Step 5:</strong> Updating campaign_id");

    $sql = "
        UPDATE email_sales_performance_tracking_over_time_report e
        JOIN (
            SELECT channel_id AS email_id, campaign_id
            FROM emma_mason_campaign_events
            WHERE type = 'email.send'
        ) c ON e.email_id = c.email_id
        SET e.campaign_id = c.campaign_id;
    ";

    if ($conn->query($sql)) {
        logCustomMessage("✅ Successfully updated campaign_id.");
    } else {
        logCustomMessage("❌ SQL Error updating campaign_id: " . $conn->error);
    }
}

// -----------------------------------------------
// Step 6: Update total_transactions_1_day and total_sales_1_day
// -----------------------------------------------
function updateTransactionsAndSalesForAllDays($conn) {
    logCustomMessage("<strong>Updating all transactions and sales columns</strong>");

    // Obtener solo los registros donde campaign_id es NULL o 0
    $query = "SELECT id, latest_order_status, DATE(first_send_date) AS first_send_date, campaign_id 
              FROM email_sales_performance_tracking_over_time_report
              WHERE campaign_id IS NULL OR campaign_id = 0";
              
    $result = $conn->query($query);
    
    if (!$result) {
        logCustomMessage("❌ SQL Error fetching report data: " . $conn->error);
        return;
    }

    while ($row = $result->fetch_assoc()) {
        $id = $row['id'];
        $latest_order_status = $row['latest_order_status'];
        $first_send_date = $row['first_send_date'];  // Convertido a DATE desde la consulta
        $campaign_id = $row['campaign_id']; // Revisamos campaign_id

        // 🚀 Validación adicional: Evita procesar si campaign_id tiene un valor diferente de NULL o 0
        if (!is_null($campaign_id) && $campaign_id != 0) {
            logCustomMessage("⏩ Skipping ID $id | campaign_id is $campaign_id");
            continue;
        }

        logCustomMessage("🔍 Processing ID $id | first_send_date: $first_send_date | latest_order_status: $latest_order_status | campaign_id: $campaign_id");

        // 🚀 Procesamos cada período de días de manera independiente
        $timeFrames = [
            "1_day" => 1,
            "5_days" => 5,
            "7_days" => 7,
            "10_days" => 10,
            "15_days" => 15
        ];

        $values = [];

        foreach ($timeFrames as $key => $days) {
            // Consulta SQL individual por cada período de días
            $countQuery = "
                SELECT COUNT(*) AS transaction_count, COALESCE(SUM(latest_order_amount), 0) AS total_sales
                FROM mautic_leads
                WHERE latest_order_status = ?
                AND DATE(latest_order_date) BETWEEN ? AND DATE_ADD(?, INTERVAL ? DAY)
            ";
            
            $stmt = $conn->prepare($countQuery);
            if (!$stmt) {
                logCustomMessage("❌ SQL Prepare Error for $key: " . $conn->error);
                continue;
            }

            $stmt->bind_param("sssi", $latest_order_status, $first_send_date, $first_send_date, $days);
            $stmt->execute();
            $stmt->bind_result($transaction_count, $total_sales);
            $stmt->fetch();
            $stmt->close();

            // Guardamos los valores en el array
            $values["total_transactions_$key"] = (int) $transaction_count;  // Ensure integer
            $values["total_sales_$key"] = (float) $total_sales;  // Ensure float
        }

        // Log para verificar los valores obtenidos antes de actualizar la base de datos
        logCustomMessage("📌 ID $id | Transactions: 1d:{$values['total_transactions_1_day']}, 5d:{$values['total_transactions_5_days']}, 7d:{$values['total_transactions_7_days']}, 10d:{$values['total_transactions_10_days']}, 15d:{$values['total_transactions_15_days']} | Sales: 1d:{$values['total_sales_1_day']}, 5d:{$values['total_sales_5_days']}, 7d:{$values['total_sales_7_days']}, 10d:{$values['total_sales_10_days']}, 15d:{$values['total_sales_15_days']}");

        // ✅ Ahora hacemos el UPDATE con los valores correctos
        $updateQuery = "
            UPDATE email_sales_performance_tracking_over_time_report 
            SET 
                total_transactions_1_day = ?, total_sales_1_day = ?, 
                total_transactions_5_days = ?, total_sales_5_days = ?, 
                total_transactions_7_days = ?, total_sales_7_days = ?, 
                total_transactions_10_days = ?, total_sales_10_days = ?, 
                total_transactions_15_days = ?, total_sales_15_days = ? 
            WHERE id = ?
        ";

        $updateStmt = $conn->prepare($updateQuery);
        if (!$updateStmt) {
            logCustomMessage("❌ SQL Prepare Error on Update: " . $conn->error);
            continue;
        }

        // 🛠 FIX: Corrected `bind_param()` to ensure the exact number of placeholders and correct types
        $updateStmt->bind_param(
            "idididididi",  // Now it has the correct number of placeholders!
            $values['total_transactions_1_day'], $values['total_sales_1_day'],
            $values['total_transactions_5_days'], $values['total_sales_5_days'],
            $values['total_transactions_7_days'], $values['total_sales_7_days'],
            $values['total_transactions_10_days'], $values['total_sales_10_days'],
            $values['total_transactions_15_days'], $values['total_sales_15_days'],
            $id
        );

        $updateStmt->execute();
        $updateStmt->close();

        logCustomMessage("✅ Updated ID $id successfully.");
    }

    logCustomMessage("✅ Finished updating all records in email_sales_performance_tracking_over_time_report.");
}




// Execute the process
populateOrUpdateReportStructure($conn);
updateFirstSendDate($conn);
updateCampaignId($conn);
updateTransactionsAndSalesForAllDays($conn);


$conn->close();
logCustomMessage("✅ <strong>Process completed successfully.</strong>");
?>
