<?php
/**
 * update_email_sales_funnel_from_staging_to_report_daily.php
 * 
 * - Processes all `email_id` in ascending order.
 * - For each `email_id`, it processes all associated `latest_order_status` values.
 * - Updates all rows in email_sales_funnel_report_daily for each email_id + latest_order_status combination.
 * - Prevents reprocessing records within 24 hours.
 * - Loops continuously until no pending records remain.
 * - Resets control columns only if 24 hours have passed since the first completed record.
 */

require_once __DIR__ . '/../config.php';
require_once __DIR__ . '/utils.php';

// -----------------------------------------------
// Database Connection
// -----------------------------------------------
$conn = new mysqli(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);
if ($conn->connect_error) {
    die("❌ Database connection error: " . $conn->connect_error);
}
$conn->set_charset("utf8mb4");

// -----------------------------------------------
// Step 1: Ensure Control Columns Exist
// -----------------------------------------------
function ensureControlColumns($conn) {
    $columnsCheck = $conn->query("SHOW COLUMNS FROM staging_sales_data LIKE 'processed_at'");
    if ($columnsCheck->num_rows == 0) {
        echo "🔧 Adding control columns to staging_sales_data...<br>";
        $sqlAlter = "
            ALTER TABLE staging_sales_data 
            ADD COLUMN processed_at DATETIME DEFAULT NULL,
            ADD COLUMN process_status ENUM('pending', 'processing', 'completed') DEFAULT 'pending'
        ";
        if (!$conn->query($sqlAlter)) {
            die("❌ Error adding control columns: " . $conn->error);
        }
    }
}
ensureControlColumns($conn);

// -----------------------------------------------
// LOOP: Process records until no more are pending
// -----------------------------------------------
while (true) {
    // 1️⃣ Get the next pending email_id in ascending order
    $sqlGetNextEmail = "
        SELECT DISTINCT email_id 
        FROM staging_sales_data
        WHERE (processed_at IS NULL OR TIMESTAMPDIFF(HOUR, processed_at, NOW()) >= 24)
          AND process_status = 'pending'
        ORDER BY email_id ASC
        LIMIT 1
    ";

    $result = $conn->query($sqlGetNextEmail);
    if (!$result) {
        die("❌ Error fetching next email_id: " . $conn->error . "<br>");
    }

    $row = $result->fetch_assoc();
    if (!$row) {
        echo "✅ No more pending email_ids to process. Exiting loop.<br><br>";
        break;  // Exit loop when no pending records
    }

    $email_id = $row['email_id'];
    echo "🔄 Processing email_id: $email_id...<br>";

    // 2️⃣ Mark as 'processing' and update processed_at
    $sqlMarkProcessing = "
        UPDATE staging_sales_data
        SET process_status = 'processing', processed_at = NOW()
        WHERE email_id = $email_id
    ";
    if (!$conn->query($sqlMarkProcessing)) {
        die("❌ Error marking email_id $email_id as processing: " . $conn->error . "<br>");
    }

    // 3️⃣ Get all latest_order_status values for this email_id
    $sqlGetStatuses = "
        SELECT DISTINCT latest_order_status 
        FROM staging_sales_data
        WHERE email_id = $email_id
    ";
    $resStatuses = $conn->query($sqlGetStatuses);
    if (!$resStatuses) {
        die("❌ Error fetching statuses for email_id $email_id: " . $conn->error . "<br>");
    }
    $statuses = [];
    while ($row = $resStatuses->fetch_assoc()) {
        $statuses[] = "'" . $conn->real_escape_string($row['latest_order_status']) . "'";
    }
    if (empty($statuses)) {
        echo "⚠️ No statuses found for email_id: $email_id<br>";
        continue;
    }
    $statusList = implode(",", $statuses);
    echo "📌 Processing latest_order_status: $statusList<br>";

    // 4️⃣ Update email_sales_funnel_report_daily with sales data from staging
    // Note: Removed the calendar_date condition so that all rows matching the combination are updated.
    $sqlUpdate = "
        UPDATE email_sales_funnel_report_daily AS d
        JOIN staging_sales_data AS s
          ON d.email_id = s.email_id
         AND d.latest_order_status = s.latest_order_status
        SET d.total_sales = s.total_sales,
            d.total_transactions = s.total_transactions
        WHERE d.email_id = $email_id
          AND d.latest_order_status IN ($statusList)
    ";
    if (!$conn->query($sqlUpdate)) {
        die("❌ Error updating email_sales_funnel_report_daily for email_id $email_id: " . $conn->error . "<br>");
    }
    echo "✅ Successfully updated records for email_id: $email_id<br>";

    // 5️⃣ Mark as 'completed' and update processed_at
    $sqlMarkCompleted = "
        UPDATE staging_sales_data
        SET process_status = 'completed', processed_at = NOW()
        WHERE email_id = $email_id
    ";
    if (!$conn->query($sqlMarkCompleted)) {
        die("❌ Error marking email_id $email_id as completed: " . $conn->error . "<br>");
    }
    echo "✅ Marked email_id: $email_id as completed.<br><br>";
}

// -----------------------------------------------
// Step 6: Reset Process Status if Everything is Done AND 24 hours have passed since the first completed record
// -----------------------------------------------
$sqlOldestProcessed = "SELECT MIN(processed_at) AS oldest_processed FROM staging_sales_data WHERE process_status = 'completed'";
$resOldest = $conn->query($sqlOldestProcessed);
$rowOldest = $resOldest->fetch_assoc();
$oldestProcessed = $rowOldest['oldest_processed'];

if ($oldestProcessed) {
    $sqlCheckRemaining = "SELECT COUNT(*) AS pending FROM staging_sales_data WHERE process_status = 'pending'";
    $resCheck = $conn->query($sqlCheckRemaining);
    $rowCheck = $resCheck->fetch_assoc();

    if ($rowCheck['pending'] == 0 && strtotime($oldestProcessed) <= strtotime('-24 hours')) {
        echo "🔄 All records processed AND 24 hours passed since the first record was completed. Resetting control columns...<br>";
        $sqlReset = "
            UPDATE staging_sales_data
            SET processed_at = NULL, process_status = 'pending'
        ";
        if (!$conn->query($sqlReset)) {
            die("❌ Error resetting control columns: " . $conn->error . "<br>");
        }
        echo "✅ Control columns reset.<br>";
    } else {
        echo "⏳ Not resetting control columns yet.<br>";
    }
}

// -----------------------------------------------
// Close connection
// -----------------------------------------------
$conn->close();
echo "🎯 Process completed successfully for all email_ids.<br>";
?>
